#include "e24api.h"

int SendCommand( HANDLE hCom, BYTE cmd, WORD par, BYTE length)
{
   BYTE tmp[3];
   DWORD ret,mask;
   tmp[0]=(par>>8)&0xFF; tmp[1]=par&0xFF; tmp[2]=cmd;
   if(length==1) tmp[0]=tmp[2];
   SetCommMask(hCom,EV_TXEMPTY);
   WriteFile(hCom,&tmp,length,&ret,NULL);
   WaitCommEvent(hCom,&mask,NULL);
   return ret;
}

int ConfigE24Chan(HANDLE hCom, BYTE mode, BYTE chan)
{
   WORD par=WORD(mode&0xF);
   return SendCommand(hCom,0x90|chan,par,3);
}

int SetGain(HANDLE hCom, BYTE gain, BYTE mode, BYTE chan)
{
   WORD par=(WORD(mode&7)<<8)|WORD(gain&7);
   return SendCommand(hCom,0xC0|chan,par,3);
}

int RefreshParam(HANDLE hCom, BYTE chan){ return SendCommand(hCom,0xD0|chan,0,1); }

int SetActiveChan(HANDLE hCom, BYTE chan){ return SendCommand(hCom,0x80|chan,0,1); }

int DigitalFilter(HANDLE hCom, BYTE mode){ return SendCommand(hCom,0xF4|(mode&1),0,1); }

int ResetCounter(HANDLE hCom){ return SendCommand(hCom,0xE0,0,1); }

int ConfigE24COM(HANDLE hCom, BYTE baud){ return SendCommand(hCom,0xE0|(baud&7),0x5A5A,3); }

double SetE24Rate(HANDLE hCom, WORD code, BYTE chan)
{
   WORD par;
   par=((code&0xF000)>>4)|((code&0x0F00)>>8);
   SendCommand(hCom,0xA0|chan,par,3);
   par=((code&0x00F0)<<4)|(code&0x000F);
   SendCommand(hCom,0xB0|chan,par,3);
   double rate=2457600/(128.0*code);
   return rate;
}

int SetEEPROMAddress(HANDLE hCom, BYTE Addr)
{
   WORD par=((WORD(Addr)&0x00F0)<<4)|(WORD(Addr)&0x000F);
   return SendCommand(hCom,0xF2,par,3);
}

BYTE GetEEPROMByte(HANDLE hCom)
{
   SendCommand(hCom,0xF1,0,1);
   unsigned char bt=0,bt1;
   DWORD ret;
   while((bt&0xF0)!=0xA0){ ReadFile(hCom,&bt,1,&ret,NULL); }
   ReadFile(hCom,&bt1,1,&ret,NULL); return (bt1<<4)|(bt&0xF);
}

int SetEEPROMByte(HANDLE hCom, BYTE bt)
{
   WORD par=((WORD(bt)&0x00F0)<<4)|(WORD(bt)&0x000F);
   unsigned char b;
   DWORD ret;
   SendCommand(hCom,0xF3,par,3);
   while((b&0xF0)!=0xB0){ ReadFile(hCom,&b,1,&ret,NULL); }
   return 1;
}

int ConvertE24Block(unsigned char *buf, long *ad_data, unsigned long *t, int *p, int *chan)
{
   *p=(buf[0]&0x40)>>6;
   *chan=(buf[0]>>3)&3;
   *ad_data=DWORD(buf[0]&7)|(DWORD(buf[1])<<3)|(DWORD(buf[2])<<10)|(DWORD(buf[3])<<17);
   *t=(DWORD(buf[5])<<7)|(DWORD(buf[6])<<14)|DWORD(buf[4]);
   *ad_data-=0x800000;
return 0;
}

int InitE24(HANDLE hCom)
{
   static DCB dcb;
   BOOL fSuccess;

   dcb.BaudRate =19200;
   dcb.ByteSize = 8;
   dcb.Parity = NOPARITY;
   dcb.StopBits = ONESTOPBIT;
   dcb.fDtrControl = DTR_CONTROL_DISABLE;
   dcb.fRtsControl = RTS_CONTROL_DISABLE;
   fSuccess = SetCommState(hCom, &dcb);
   if(!fSuccess) return 1;     // fail

   PurgeComm(hCom,PURGE_TXCLEAR|PURGE_RXCLEAR);

   dcb.BaudRate =19200;
   dcb.ByteSize = 8;
   dcb.Parity = NOPARITY;
   dcb.StopBits = ONESTOPBIT;
   dcb.fDtrControl = DTR_CONTROL_DISABLE;
   dcb.fRtsControl = RTS_CONTROL_ENABLE;
   fSuccess = SetCommState(hCom, &dcb);
   if(!fSuccess) return 1;   // fail

   unsigned char data[7];
   DWORD ret=0;

   ReadFile(hCom,&data,7,&ret,NULL); // we wait while E24 loaded from EEPROM and start work

   return ret;  // return 7 is ok
}
